@@ -8,6 +8,7 @@ import (
 	"regexp"
 
 	_ "github.com/uber/athenadriver/go" // DRIVER: awsathena
+	"github.com/xo/dburl"
 	"github.com/xo/usql/drivers"
 )
 
@@ -15,7 +16,7 @@ func init() {
 	endRE := regexp.MustCompile(`;?\s*$`)
 	drivers.Register("awsathena", drivers.Driver{
 		AllowMultilineComments: true,
-		Process: func(prefix string, sqlstr string) (string, string, bool, error) {
+		Process: func(_ *dburl.URL, prefix string, sqlstr string) (string, string, bool, error) {
 			sqlstr = endRE.ReplaceAllString(sqlstr, "")
 			typ, q := drivers.QueryExecType(prefix, sqlstr)
 			return typ, sqlstr, q, nil
