@@ -74,7 +74,9 @@ type Driver struct {
 	// IsPasswordErr will be used by IsPasswordErr if defined.
 	IsPasswordErr func(error) bool
 	// Process will be used by Process if defined.
-	Process func(string, string) (string, string, bool, error)
+	Process func(*dburl.URL, string, string) (string, string, bool, error)
+	// ColumnTypes is a callback that will be used if
+	ColumnTypes func(*sql.ColumnType) (interface{}, error)
 	// RowsAffected will be used by RowsAffected if defined.
 	RowsAffected func(sql.Result) (int64, error)
 	// Err will be used by Error.Error if defined.
@@ -246,13 +248,18 @@ func User(ctx context.Context, u *dburl.URL, db DB) (string, error) {
 // Process processes the sql query for a driver.
 func Process(u *dburl.URL, prefix, sqlstr string) (string, string, bool, error) {
 	if d, ok := drivers[u.Driver]; ok && d.Process != nil {
-		a, b, c, err := d.Process(prefix, sqlstr)
+		a, b, c, err := d.Process(u, prefix, sqlstr)
 		return a, b, c, WrapErr(u.Driver, err)
 	}
 	typ, q := QueryExecType(prefix, sqlstr)
 	return typ, sqlstr, q, nil
 }
 
+// ColumnTypes returns the column types callback for a driver.
+func ColumnTypes(u *dburl.URL) func(*sql.ColumnType) (interface{}, error) {
+	return drivers[u.Driver].ColumnTypes
+}
+
 // IsPasswordErr returns true if an err is a password error for a driver.
 func IsPasswordErr(u *dburl.URL, err error) bool {
 	drv := u.Driver
@@ -533,16 +540,12 @@ func CopyWithInsert(placeholder func(int) string) func(ctx context.Context, db *
 		if !strings.HasPrefix(strings.ToLower(query), "insert into") {
 			leftParen := strings.IndexRune(table, '(')
 			if leftParen == -1 {
-				colStmt, err := db.PrepareContext(ctx, "SELECT * FROM "+table+" WHERE 1=0")
-				if err != nil {
-					return 0, fmt.Errorf("failed to prepare query to determine target table columns: %w", err)
-				}
-				defer colStmt.Close()
-				colRows, err := colStmt.QueryContext(ctx)
+				colRows, err := db.QueryContext(ctx, "SELECT * FROM "+table+" WHERE 1=0")
 				if err != nil {
 					return 0, fmt.Errorf("failed to execute query to determine target table columns: %w", err)
 				}
 				columns, err := colRows.Columns()
+				_ = colRows.Close()
 				if err != nil {
 					return 0, fmt.Errorf("failed to fetch target table columns: %w", err)
 				}
@@ -569,8 +572,11 @@ func CopyWithInsert(placeholder func(int) string) func(ctx context.Context, db *
 			return 0, fmt.Errorf("failed to fetch source column types: %w", err)
 		}
 		values := make([]interface{}, clen)
+		valueRefs := make([]reflect.Value, clen)
+		actuals := make([]interface{}, clen)
 		for i := 0; i < len(columnTypes); i++ {
-			values[i] = reflect.New(columnTypes[i].ScanType()).Interface()
+			valueRefs[i] = reflect.New(columnTypes[i].ScanType())
+			values[i] = valueRefs[i].Interface()
 		}
 		var n int64
 		for rows.Next() {
@@ -578,7 +584,12 @@ func CopyWithInsert(placeholder func(int) string) func(ctx context.Context, db *
 			if err != nil {
 				return n, fmt.Errorf("failed to scan row: %w", err)
 			}
-			res, err := stmt.ExecContext(ctx, values...)
+			//We can't use values... in Exec() below, because some drivers
+			//don't accept pointer to an argument instead of the arg itself.
+			for i := range values {
+				actuals[i] = valueRefs[i].Elem().Interface()
+			}
+			res, err := stmt.ExecContext(ctx, actuals...)
 			if err != nil {
 				return n, fmt.Errorf("failed to exec insert: %w", err)
 			}
@@ -597,3 +608,7 @@ func CopyWithInsert(placeholder func(int) string) func(ctx context.Context, db *
 		return n, rows.Err()
 	}
 }
+
+func init() {
+	dburl.OdbcIgnoreQueryPrefixes = []string{"usql_"}
+}
