@@ -68,6 +68,8 @@ type Handler struct {
 	// batch
 	batch    bool
 	batchEnd string
+	// bind are bound values for exec statements
+	bind []interface{}
 	// connection
 	u  *dburl.URL
 	db *sql.DB
@@ -100,6 +102,7 @@ func New(l rline.IO, user *user.User, wd string, nopw bool) *Handler {
 	}
 	if iactive {
 		l.SetOutput(h.outputHighlighter)
+		l.Completer(completer.NewDefaultCompleter(completer.WithConnStrings(h.connStrings())))
 	}
 	return h
 }
@@ -225,9 +228,16 @@ var helpQuitExitRE = regexp.MustCompile(fmt.Sprintf(`(?im)^(%s|%s|%s)\s*$`, text
 func (h *Handler) Run() error {
 	stdout, stderr, iactive := h.l.Stdout(), h.l.Stderr(), h.l.Interactive()
 	// display welcome info
-	if iactive {
-		fmt.Fprintln(h.l.Stdout(), text.WelcomeDesc)
-		fmt.Fprintln(h.l.Stdout())
+	if iactive && env.Get("QUIET") == "off" {
+		// logo
+		if typ := env.TermGraphics(); typ.Available() {
+			if err := typ.Encode(stdout, text.Logo); err != nil {
+				return err
+			}
+		}
+		// welcome text
+		fmt.Fprintln(stdout, text.WelcomeDesc)
+		fmt.Fprintln(stdout)
 	}
 	var lastErr error
 	for {
@@ -374,7 +384,7 @@ func (h *Handler) Run() error {
 					out = h.out
 				}
 				ctx, stop := signal.NotifyContext(context.Background(), os.Interrupt)
-				if err = h.Execute(ctx, out, opt, h.lastPrefix, h.last, forceBatch); err != nil {
+				if err = h.Execute(ctx, out, opt, h.lastPrefix, h.last, forceBatch, h.unbind()...); err != nil {
 					lastErr = WrapErr(h.last, err)
 					if env.All()["ON_ERROR_STOP"] == "on" {
 						if iactive {
@@ -396,7 +406,7 @@ func (h *Handler) Run() error {
 }
 
 // Execute executes a query against the connected database.
-func (h *Handler) Execute(ctx context.Context, w io.Writer, opt metacmd.Option, prefix, sqlstr string, forceTrans bool) error {
+func (h *Handler) Execute(ctx context.Context, w io.Writer, opt metacmd.Option, prefix, sqlstr string, forceTrans bool, bind ...interface{}) error {
 	if h.db == nil {
 		return text.ErrNotConnected
 	}
@@ -411,16 +421,16 @@ func (h *Handler) Execute(ctx context.Context, w io.Writer, opt metacmd.Option,
 			return err
 		}
 	}
-	f := h.execSingle
+	f := h.doExecSingle
 	switch opt.Exec {
 	case metacmd.ExecExec:
-		f = h.execExec
+		f = h.doExecExec
 	case metacmd.ExecSet:
-		f = h.execSet
+		f = h.doExecSet
 	case metacmd.ExecWatch:
-		f = h.execWatch
+		f = h.doExecWatch
 	}
-	if err = drivers.WrapErr(h.u.Driver, f(ctx, w, opt, prefix, sqlstr, qtyp)); err != nil {
+	if err = drivers.WrapErr(h.u.Driver, f(ctx, w, opt, prefix, sqlstr, qtyp, bind)); err != nil {
 		if forceTrans {
 			defer h.tx.Rollback()
 			h.tx = nil
@@ -439,6 +449,18 @@ func (h *Handler) Reset(r []rune) {
 	h.last, h.lastPrefix, h.lastRaw, h.batch, h.batchEnd = "", "", "", false, ""
 }
 
+// Bind sets the bind parameters for the next query execution.
+func (h *Handler) Bind(bind []interface{}) {
+	h.bind = bind
+}
+
+// unbind returns the bind parameters.
+func (h *Handler) unbind() []interface{} {
+	v := h.bind
+	h.bind = nil
+	return v
+}
+
 // Prompt parses a prompt.
 //
 // NOTE: the documentation below is INCORRECT, as it is just copied from
@@ -711,19 +733,21 @@ func (h *Handler) Highlight(w io.Writer, buf string) error {
 // an appropriate driver (mysql, postgres, sqlite3) depending on the type (unix
 // domain socket, directory, or regular file, respectively).
 func (h *Handler) Open(ctx context.Context, params ...string) error {
-	// build a list of all possible connStrings for the completer
-	connStrings := h.connStrings()
 	if len(params) == 0 || params[0] == "" {
-		h.l.Completer(completer.NewDefaultCompleter(completer.WithConnStrings(connStrings)))
 		return nil
 	}
 	if h.tx != nil {
 		return text.ErrPreviousTransactionExists
 	}
+	if len(params) == 1 {
+		if v, ok := env.Cget(params[0]); ok {
+			params = v
+		}
+	}
 	if len(params) < 2 {
-		urlstr := params[0]
+		dsn := params[0]
 		// parse dsn
-		u, err := dburl.Parse(urlstr)
+		u, err := dburl.Parse(dsn)
 		if err != nil {
 			return err
 		}
@@ -738,7 +762,7 @@ func (h *Handler) Open(ctx context.Context, params ...string) error {
 	}
 	// open connection
 	var err error
-	h.db, err = drivers.Open(ctx, h.u, h.GetOutput, h.IO().Stderr)
+	h.db, err = drivers.Open(ctx, h.u, h.GetOutput, h.l.Stderr)
 	if err != nil && !drivers.IsPasswordErr(h.u, err) {
 		defer h.Close()
 		return err
@@ -748,7 +772,6 @@ func (h *Handler) Open(ctx context.Context, params ...string) error {
 	// force error/check connection
 	if err == nil {
 		if err = drivers.Ping(ctx, h.u, h.db); err == nil {
-			h.l.Completer(drivers.NewCompleter(ctx, h.u, h.db, readerOpts(), completer.WithConnStrings(connStrings)))
 			return h.Version(ctx)
 		}
 	}
@@ -804,6 +827,9 @@ func (h *Handler) connStrings() []string {
 		}
 		names = append(names, fmt.Sprintf("%s://%s%s%s%s", entry.Protocol, user, host, port, dbname))
 	}
+	for name := range env.Call() {
+		names = append(names, name)
+	}
 	sort.Strings(names)
 	return names
 }
@@ -830,8 +856,13 @@ func (h *Handler) forceParams(u *dburl.URL) {
 // Password collects a password from input, and returns a modified DSN
 // including the collected password.
 func (h *Handler) Password(dsn string) (string, error) {
-	if dsn == "" {
+	switch v, ok := env.Cget(dsn); {
+	case dsn == "":
 		return "", text.ErrMissingDSN
+	case ok && len(v) < 2:
+		return "", text.ErrNamedConnectionIsNotAURL
+	case ok:
+		dsn = v[0]
 	}
 	u, err := dburl.Parse(dsn)
 	if err != nil {
@@ -954,19 +985,20 @@ func (h *Handler) ChangePassword(user string) (string, error) {
 
 // Version prints the database version information after a successful connection.
 func (h *Handler) Version(ctx context.Context) error {
-	if env.Get("SHOW_HOST_INFORMATION") != "true" {
+	if env.Get("SHOW_HOST_INFORMATION") != "true" || !h.l.Interactive() {
 		return nil
 	}
 	if h.db == nil {
 		return text.ErrNotConnected
 	}
 	ver, err := drivers.Version(ctx, h.u, h.DB())
-	if err != nil {
+	switch {
+	case err != nil:
 		ver = fmt.Sprintf("<unknown, error: %v>", err)
+	case ver == "":
+		ver = "<unknown>"
 	}
-	if ver != "" {
-		h.Print(text.ConnInfo, h.u.Driver, ver)
-	}
+	h.Print(text.ConnInfo, h.u.Driver, ver)
 	return nil
 }
 
@@ -978,14 +1010,14 @@ func (h *Handler) Print(format string, a ...interface{}) {
 	fmt.Fprintln(h.l.Stdout(), fmt.Sprintf(format, a...))
 }
 
-// execWatch repeatedly executes a query against the database.
-func (h *Handler) execWatch(ctx context.Context, w io.Writer, opt metacmd.Option, prefix, sqlstr string, qtyp bool) error {
+// doExecWatch repeatedly executes a query against the database.
+func (h *Handler) doExecWatch(ctx context.Context, w io.Writer, opt metacmd.Option, prefix, sqlstr string, qtyp bool, bind []interface{}) error {
 	for {
 		// this is the actual output that psql has: "Mon Jan 2006 3:04:05 PM MST"
 		// fmt.Fprintf(w, "%s (every %fs)\n\n", time.Now().Format("Mon Jan 2006 3:04:05 PM MST"), float64(opt.Watch)/float64(time.Second))
 		fmt.Fprintf(w, "%s (every %v)\n", time.Now().Format(time.RFC1123), opt.Watch)
 		fmt.Fprintln(w)
-		if err := h.execSingle(ctx, w, opt, prefix, sqlstr, qtyp); err != nil {
+		if err := h.doExecSingle(ctx, w, opt, prefix, sqlstr, qtyp, bind); err != nil {
 			return err
 		}
 		select {
@@ -999,21 +1031,35 @@ func (h *Handler) execWatch(ctx context.Context, w io.Writer, opt metacmd.Option
 	}
 }
 
-// execSingle executes a single query against the database based on its query type.
-func (h *Handler) execSingle(ctx context.Context, w io.Writer, opt metacmd.Option, prefix, sqlstr string, qtyp bool) error {
+// doExecSingle executes a single query against the database based on its query type.
+func (h *Handler) doExecSingle(ctx context.Context, w io.Writer, opt metacmd.Option, prefix, sqlstr string, qtyp bool, bind []interface{}) error {
 	// exec or query
-	f := h.exec
+	f := h.doExec
 	if qtyp {
-		f = h.query
+		f = h.doQuery
 	}
 	// exec
-	return f(ctx, w, opt, prefix, sqlstr)
+	start := time.Now()
+	if err := f(ctx, w, opt, prefix, sqlstr, bind); err != nil {
+		return err
+	}
+	if h.timing {
+		d := time.Since(start)
+		s := text.TimingDesc
+		v := []interface{}{float64(d.Microseconds()) / 1000}
+		if d > 1*time.Second {
+			s += " (%v)"
+			v = append(v, d.Round(1*time.Millisecond))
+		}
+		fmt.Fprintln(h.l.Stdout(), fmt.Sprintf(s, v...))
+	}
+	return nil
 }
 
-// execSet executes a SQL query, setting all returned columns as variables.
-func (h *Handler) execSet(ctx context.Context, w io.Writer, opt metacmd.Option, prefix, sqlstr string, _ bool) error {
+// doExecSet executes a SQL query, setting all returned columns as variables.
+func (h *Handler) doExecSet(ctx context.Context, w io.Writer, opt metacmd.Option, prefix, sqlstr string, _ bool, bind []interface{}) error {
 	// query
-	rows, err := h.DB().QueryContext(ctx, sqlstr)
+	rows, err := h.DB().QueryContext(ctx, sqlstr, bind...)
 	if err != nil {
 		return err
 	}
@@ -1049,32 +1095,31 @@ func (h *Handler) execSet(ctx context.Context, w io.Writer, opt metacmd.Option,
 	return nil
 }
 
-// execExec executes a query and re-executes all columns of all rows as if they
+// doExecExec executes a query and re-executes all columns of all rows as if they
 // were their own queries.
-func (h *Handler) execExec(ctx context.Context, w io.Writer, _ metacmd.Option, prefix, sqlstr string, qtyp bool) error {
+func (h *Handler) doExecExec(ctx context.Context, w io.Writer, _ metacmd.Option, prefix, sqlstr string, qtyp bool, bind []interface{}) error {
 	// query
-	rows, err := h.DB().QueryContext(ctx, sqlstr)
+	rows, err := h.DB().QueryContext(ctx, sqlstr, bind...)
 	if err != nil {
 		return err
 	}
-	// execRows
-	if err := h.execRows(ctx, w, rows); err != nil {
+	// exec resulting rows
+	if err := h.doExecRows(ctx, w, rows); err != nil {
 		return err
 	}
 	// check for additional result sets ...
 	for rows.NextResultSet() {
-		if err := h.execRows(ctx, w, rows); err != nil {
+		if err := h.doExecRows(ctx, w, rows); err != nil {
 			return err
 		}
 	}
 	return nil
 }
 
-// query executes a query against the database.
-func (h *Handler) query(ctx context.Context, w io.Writer, opt metacmd.Option, typ, sqlstr string) error {
-	start := time.Now()
+// doQuery executes a doQuery against the database.
+func (h *Handler) doQuery(ctx context.Context, w io.Writer, opt metacmd.Option, typ, sqlstr string, bind []interface{}) error {
 	// run query
-	rows, err := h.DB().QueryContext(ctx, sqlstr)
+	rows, err := h.DB().QueryContext(ctx, sqlstr, bind...)
 	if err != nil {
 		return err
 	}
@@ -1093,7 +1138,7 @@ func (h *Handler) query(ctx context.Context, w io.Writer, opt metacmd.Option, ty
 		}
 		if pipeName != "" {
 			if pipeName[0] == '|' {
-				pipe, cmd, err = env.Pipe(pipeName[1:])
+				pipe, cmd, err = env.Pipe(h.l.Stdout(), h.l.Stderr(), pipeName[1:])
 			} else {
 				pipe, err = os.OpenFile(pipeName, os.O_TRUNC|os.O_CREATE|os.O_WRONLY, 0o644)
 			}
@@ -1105,25 +1150,29 @@ func (h *Handler) query(ctx context.Context, w io.Writer, opt metacmd.Option, ty
 	} else if opt.Exec != metacmd.ExecWatch {
 		params["pager_cmd"] = env.All()["PAGER"]
 	}
-	useColumnTypes := drivers.UseColumnTypes(h.u)
+	// set up column type config
+	var extra []tblfmt.Option
+	switch f := drivers.ColumnTypes(h.u); {
+	case f != nil:
+		extra = append(extra, tblfmt.WithColumnTypesFunc(f))
+	case drivers.UseColumnTypes(h.u):
+		extra = append(extra, tblfmt.WithUseColumnTypes(true))
+	}
 	// wrap query with crosstab
 	resultSet := tblfmt.ResultSet(rows)
 	if opt.Exec == metacmd.ExecCrosstab {
 		var err error
-		resultSet, err = tblfmt.NewCrosstabView(rows, tblfmt.WithParams(opt.Crosstab...), tblfmt.WithUseColumnTypes(useColumnTypes))
+		resultSet, err = tblfmt.NewCrosstabView(rows, append(extra, tblfmt.WithParams(opt.Crosstab...))...)
 		if err != nil {
 			return err
 		}
-		useColumnTypes = false
+		extra = nil
 	}
 	if drivers.LowerColumnNames(h.u) {
 		params["lower_column_names"] = "true"
 	}
-	if useColumnTypes {
-		params["use_column_types"] = "true"
-	}
 	// encode and handle error conditions
-	switch err := tblfmt.EncodeAll(w, resultSet, params); {
+	switch err := tblfmt.EncodeAll(w, resultSet, params, extra...); {
 	case err != nil && cmd != nil && errors.Is(err, syscall.EPIPE):
 		// broken pipe means pager quit before consuming all data, which might be expected
 		return nil
@@ -1136,16 +1185,6 @@ func (h *Handler) query(ctx context.Context, w io.Writer, opt metacmd.Option, ty
 	case params["format"] == "aligned":
 		fmt.Fprintln(w)
 	}
-	if h.timing {
-		d := time.Since(start)
-		format := text.TimingDesc
-		v := []interface{}{float64(d.Microseconds()) / 1000}
-		if d > 1*time.Second {
-			format += " (%v)"
-			v = append(v, d.Round(1*time.Millisecond))
-		}
-		h.Print(format, v...)
-	}
 	if pipe != nil {
 		pipe.Close()
 		if cmd != nil {
@@ -1155,8 +1194,8 @@ func (h *Handler) query(ctx context.Context, w io.Writer, opt metacmd.Option, ty
 	return err
 }
 
-// execRows executes all the columns in the row.
-func (h *Handler) execRows(ctx context.Context, w io.Writer, rows *sql.Rows) error {
+// doExecRows executes all the columns in the row.
+func (h *Handler) doExecRows(ctx context.Context, w io.Writer, rows *sql.Rows) error {
 	// get columns
 	cols, err := drivers.Columns(h.u, rows)
 	if err != nil {
@@ -1237,9 +1276,9 @@ func (h *Handler) scan(rows *sql.Rows, clen int, tfmt string) ([]string, error)
 	return row, nil
 }
 
-// exec does a database exec.
-func (h *Handler) exec(ctx context.Context, w io.Writer, _ metacmd.Option, typ, sqlstr string) error {
-	res, err := h.DB().ExecContext(ctx, sqlstr)
+// doExec does a database exec.
+func (h *Handler) doExec(ctx context.Context, w io.Writer, _ metacmd.Option, typ, sqlstr string, bind []interface{}) error {
+	res, err := h.DB().ExecContext(ctx, sqlstr, bind...)
 	if err != nil {
 		_ = env.Set("ROW_COUNT", "0")
 		return err
@@ -1251,12 +1290,14 @@ func (h *Handler) exec(ctx context.Context, w io.Writer, _ metacmd.Option, typ,
 		return err
 	}
 	// print name
-	fmt.Fprint(w, typ)
-	// print count
-	if count > 0 {
-		fmt.Fprint(w, " ", count)
+	if env.Get("QUIET") == "off" {
+		fmt.Fprint(w, typ)
+		// print count
+		if count > 0 {
+			fmt.Fprint(w, " ", count)
+		}
+		fmt.Fprintln(w)
 	}
-	fmt.Fprintln(w)
 	return env.Set("ROW_COUNT", strconv.FormatInt(count, 10))
 }
 
@@ -1392,6 +1433,32 @@ func (h *Handler) SetOutput(o io.WriteCloser) {
 	h.out = o
 }
 
+// Error wraps handler errors.
+type Error struct {
+	Buf string
+	Err error
+}
+
+// WrapErr wraps an [error] using the specified driver when err is not nil.
+func WrapErr(buf string, err error) error {
+	if err == nil {
+		return nil
+	}
+	// avoid double wrapping error
+	if _, ok := err.(*Error); ok {
+		return err
+	}
+	return &Error{buf, err}
+}
+
+// Error satisfies the [error] interface, returning the original error message.
+func (e *Error) Error() string {
+	return e.Err.Error()
+}
+
+// Unwrap returns the original error.
+func (e *Error) Unwrap() error { return e.Err }
+
 func readerOpts() []metadata.ReaderOption {
 	var opts []metadata.ReaderOption
 	envs := env.All()
@@ -1448,3 +1515,27 @@ func grab(r []rune, i, end int) rune {
 	}
 	return 0
 }
+
+// linetermRE is the end of line terminal.
+var linetermRE = regexp.MustCompile(`(?:\r?\n)+$`)
+
+// empty reports whether s contains at least one printable, non-space character.
+func empty(s string) bool {
+	i := strings.IndexFunc(s, func(r rune) bool {
+		return unicode.IsPrint(r) && !unicode.IsSpace(r)
+	})
+	return i == -1
+}
+
+var ansiRE = regexp.MustCompile(`\x1b[[0-9]+([:;][0-9]+)*m`)
+
+// lastcolor returns the last defined color in s, if any.
+func lastcolor(s string) string {
+	if i := strings.LastIndex(s, "\n"); i != -1 {
+		s = s[:i]
+	}
+	if i := strings.LastIndex(s, "\x1b[0m"); i != -1 {
+		s = s[i+4:]
+	}
+	return strings.Join(ansiRE.FindAllString(s, -1), "")
+}
