@@ -7,17 +7,18 @@ import (
 	"fmt"
 	"io"
 	"os"
+	"os/exec"
 	"os/signal"
 	"sort"
 	"strconv"
 	"strings"
 	"time"
 
-	"github.com/alecthomas/kingpin/v2"
 	"github.com/xo/dburl"
 	"github.com/xo/usql/drivers"
 	"github.com/xo/usql/env"
 	"github.com/xo/usql/text"
+	"golang.org/x/exp/maps"
 )
 
 // Cmd is a command implementation.
@@ -53,16 +54,28 @@ func init() {
 				if err != nil {
 					return err
 				}
-				switch name {
+				stdout, stderr := p.Handler.IO().Stdout(), p.Handler.IO().Stderr()
+				var cmd *exec.Cmd
+				var wc io.WriteCloser
+				if pager := env.Get("PAGER"); p.Handler.IO().Interactive() && pager != "" {
+					if wc, cmd, err = env.Pipe(stdout, stderr, pager); err != nil {
+						return err
+					}
+					stdout = wc
+				}
+				switch name = strings.TrimSpace(strings.ToLower(name)); {
+				case name == "options":
+					Usage(stdout, true)
+				case name == "variables":
+					env.Listing(stdout)
 				default:
-					Listing(p.Handler.IO().Stdout())
-				case "commands":
-					Listing(p.Handler.IO().Stdout())
-				case "options":
-					// FIXME: decouple
-					kingpin.Usage()
-				case "variables":
-					env.Listing(p.Handler.IO().Stdout())
+					Listing(stdout)
+				}
+				if cmd != nil {
+					if err := wc.Close(); err != nil {
+						return err
+					}
+					return cmd.Wait()
 				}
 				return nil
 			},
@@ -82,7 +95,11 @@ func init() {
 			Name:    "copyright",
 			Desc:    Desc{"show " + text.CommandName + " usage and distribution terms", ""},
 			Process: func(p *Params) error {
-				p.Handler.Print(text.Copyright)
+				stdout := p.Handler.IO().Stdout()
+				if typ := env.TermGraphics(); typ.Available() {
+					typ.Encode(stdout, text.Logo)
+				}
+				fmt.Fprintln(stdout, text.Copyright)
 				return nil
 			},
 		},
@@ -91,11 +108,11 @@ func init() {
 			Name:    "conninfo",
 			Desc:    Desc{"display information about the current database connection", ""},
 			Process: func(p *Params) error {
+				s := text.NotConnected
 				if db, u := p.Handler.DB(), p.Handler.URL(); db != nil && u != nil {
-					p.Handler.Print(text.ConnInfo, u.Driver, u.DSN)
-				} else {
-					p.Handler.Print(text.NotConnected)
+					s = fmt.Sprintf(text.ConnInfo, u.Driver, u.DSN)
 				}
+				fmt.Fprintln(p.Handler.IO().Stdout(), s)
 				return nil
 			},
 		},
@@ -104,7 +121,16 @@ func init() {
 			Name:    "drivers",
 			Desc:    Desc{"display information about available database drivers", ""},
 			Process: func(p *Params) error {
-				out := p.Handler.IO().Stdout()
+				stdout, stderr := p.Handler.IO().Stdout(), p.Handler.IO().Stderr()
+				var cmd *exec.Cmd
+				var wc io.WriteCloser
+				if pager := env.Get("PAGER"); p.Handler.IO().Interactive() && pager != "" {
+					var err error
+					if wc, cmd, err = env.Pipe(stdout, stderr, pager); err != nil {
+						return err
+					}
+					stdout = wc
+				}
 				available := drivers.Available()
 				names := make([]string, len(available))
 				var z int
@@ -113,7 +139,7 @@ func init() {
 					z++
 				}
 				sort.Strings(names)
-				fmt.Fprintln(out, text.AvailableDrivers)
+				fmt.Fprintln(stdout, text.AvailableDrivers)
 				for _, n := range names {
 					s := "  " + n
 					driver, aliases := dburl.SchemeDriverAndAliases(n)
@@ -125,7 +151,13 @@ func init() {
 							s += " [" + strings.Join(aliases, ", ") + "]"
 						}
 					}
-					fmt.Fprintln(out, s)
+					fmt.Fprintln(stdout, s)
+				}
+				if cmd != nil {
+					if err := wc.Close(); err != nil {
+						return err
+					}
+					return cmd.Wait()
 				}
 				return nil
 			},
@@ -148,6 +180,35 @@ func init() {
 				return p.Handler.Open(ctx, vals...)
 			},
 		},
+		SetConnVar: {
+			Section: SectionConnection,
+			Name:    "cset",
+			Desc:    Desc{"set named connection, or list all if no parameters", "[NAME [DSN]]"},
+			Aliases: map[string]Desc{
+				"cset": {"define named connection for database driver", "NAME DRIVER PARAMS..."},
+			},
+			Process: func(p *Params) error {
+				ok, n, err := p.GetOK(true)
+				switch {
+				case err != nil:
+					return err
+				case ok:
+					vals, err := p.GetAll(true)
+					if err != nil {
+						return err
+					}
+					return env.Cset(n, vals...)
+				}
+				vals := env.Call()
+				keys := maps.Keys(vals)
+				sort.Strings(keys)
+				out := p.Handler.IO().Stdout()
+				for _, k := range keys {
+					fmt.Fprintln(out, k, "=", "'"+strings.Join(vals[k], " ")+"'")
+				}
+				return nil
+			},
+		},
 		Disconnect: {
 			Section: SectionConnection,
 			Name:    "Z",
@@ -257,6 +318,26 @@ func init() {
 				return nil
 			},
 		},
+		Bind: {
+			Section: SectionQueryExecute,
+			Name:    "bind",
+			Desc:    Desc{"set query parameters", "[PARAM]..."},
+			Process: func(p *Params) error {
+				bind, err := p.GetAll(true)
+				if err != nil {
+					return err
+				}
+				var v []interface{}
+				if n := len(bind); n != 0 {
+					v = make([]interface{}, len(bind))
+					for i := 0; i < n; i++ {
+						v[i] = bind[i]
+					}
+				}
+				p.Handler.Bind(v)
+				return nil
+			},
+		},
 		Edit: {
 			Section: SectionQueryBuffer,
 			Name:    "e",
@@ -331,7 +412,7 @@ func init() {
 			Aliases: map[string]Desc{"reset": {}},
 			Process: func(p *Params) error {
 				p.Handler.Reset(nil)
-				fmt.Fprintln(p.Handler.IO().Stdout(), text.QueryBufferReset)
+				p.Handler.Print(text.QueryBufferReset)
 				return nil
 			},
 		},
@@ -344,30 +425,32 @@ func init() {
 				"warn":  {"write string to standard error (-n for no newline)", "[-n] [STRING]"},
 			},
 			Process: func(p *Params) error {
-				nl := "\n"
-				var vals []string
 				ok, n, err := p.GetOptional(true)
 				if err != nil {
 					return err
 				}
-				if ok && n == "n" {
-					nl = ""
-				} else if ok {
+				f := fmt.Fprintln
+				var vals []string
+				switch {
+				case ok && n == "n":
+					f = fmt.Fprint
+				case ok:
 					vals = append(vals, "-"+n)
-				} else {
+				default:
 					vals = append(vals, n)
 				}
 				v, err := p.GetAll(true)
 				if err != nil {
 					return err
 				}
-				out := io.Writer(p.Handler.IO().Stdout())
-				if o := p.Handler.GetOutput(); p.Name == "qecho" && o != nil {
+				out := p.Handler.IO().Stdout()
+				switch o := p.Handler.GetOutput(); {
+				case p.Name == "qecho" && o != nil:
 					out = o
-				} else if p.Name == "warn" {
+				case p.Name == "warn":
 					out = p.Handler.IO().Stderr()
 				}
-				fmt.Fprint(out, strings.Join(append(vals, v...), " ")+nl)
+				f(out, strings.Join(append(vals, v...), " "))
 				return nil
 			},
 		},
@@ -401,6 +484,29 @@ func init() {
 				return env.Chdir(p.Handler.User(), dir)
 			},
 		},
+		GetEnv: {
+			Section: SectionOperatingSystem,
+			Name:    "getenv",
+			Desc:    Desc{"fetch environment variable", "VARNAME ENVVAR"},
+			Process: func(p *Params) error {
+				n, err := p.Get(true)
+				switch {
+				case err != nil:
+					return err
+				case n == "":
+					return text.ErrMissingRequiredArgument
+				}
+				v, err := p.Get(true)
+				switch {
+				case err != nil:
+					return err
+				case v == "":
+					return text.ErrMissingRequiredArgument
+				}
+				value, _ := env.Getenv(v)
+				return env.Set(n, value)
+			},
+		},
 		SetEnv: {
 			Section: SectionOperatingSystem,
 			Name:    "setenv",
@@ -463,9 +569,7 @@ func init() {
 			Desc:    Desc{"send all query results to file or |pipe", "[FILE]"},
 			Aliases: map[string]Desc{"out": {}},
 			Process: func(p *Params) error {
-				if out := p.Handler.GetOutput(); out != nil {
-					p.Handler.SetOutput(nil)
-				}
+				p.Handler.SetOutput(nil)
 				params, err := p.GetAll(true)
 				if err != nil {
 					return err
@@ -476,7 +580,7 @@ func init() {
 				}
 				var out io.WriteCloser
 				if pipe[0] == '|' {
-					out, _, err = env.Pipe(pipe[1:])
+					out, _, err = env.Pipe(p.Handler.IO().Stdout(), p.Handler.IO().Stderr(), pipe[1:])
 				} else {
 					out, err = os.OpenFile(pipe, os.O_TRUNC|os.O_CREATE|os.O_WRONLY, 0o644)
 				}
@@ -572,7 +676,7 @@ func init() {
 		Prompt: {
 			Section: SectionVariables,
 			Name:    "prompt",
-			Desc:    Desc{"prompt user to set variable", "[-TYPE] <VAR> [PROMPT]"},
+			Desc:    Desc{"prompt user to set variable", "[-TYPE] VAR [PROMPT]"},
 			Process: func(p *Params) error {
 				typ := "string"
 				ok, n, err := p.GetOptional(true)
@@ -609,29 +713,24 @@ func init() {
 			Desc:    Desc{"set internal variable, or list all if no parameters", "[NAME [VALUE]]"},
 			Process: func(p *Params) error {
 				ok, n, err := p.GetOK(true)
-				if err != nil {
+				switch {
+				case err != nil:
 					return err
-				}
-				if !ok {
-					vals := env.All()
-					out := p.Handler.IO().Stdout()
-					n := make([]string, len(vals))
-					var i int
-					for k := range vals {
-						n[i] = k
-						i++
+				case ok:
+					vals, err := p.GetAll(true)
+					if err != nil {
+						return err
 					}
-					sort.Strings(n)
-					for _, k := range n {
-						fmt.Fprintln(out, k, "=", "'"+vals[k]+"'")
-					}
-					return nil
+					return env.Set(n, strings.Join(vals, " "))
 				}
-				vals, err := p.GetAll(true)
-				if err != nil {
-					return err
+				vals := env.All()
+				keys := maps.Keys(vals)
+				sort.Strings(keys)
+				out := p.Handler.IO().Stdout()
+				for _, k := range keys {
+					fmt.Fprintln(out, k, "=", "'"+vals[k]+"'")
 				}
-				return env.Set(n, strings.Join(vals, ""))
+				return nil
 			},
 		},
 		Unset: {
@@ -646,7 +745,7 @@ func init() {
 				return env.Unset(n)
 			},
 		},
-		SetFormatVar: {
+		SetPrintVar: {
 			Section: SectionFormatting,
 			Name:    "pset",
 			Desc:    Desc{"set table output option", "[NAME [VALUE]]"},
@@ -921,3 +1020,7 @@ func init() {
 		sectMap[c.Section] = append(sectMap[c.Section], mc)
 	}
 }
+
+// Usage is used by the [Question] command to display command line options.
+var Usage = func(io.Writer, bool) {
+}
