@@ -3,6 +3,10 @@
 package text
 
 import (
+	"bytes"
+	_ "embed"
+	"image"
+	"image/png"
 	"regexp"
 	"strings"
 )
@@ -12,7 +16,9 @@ var (
 	CommandName           = `usql`
 	CommandVersion        = `0.0.0-dev`
 	PassfileName          = CommandName + `pass`
+	ConfigName            = "config"
 	Banner                = `the universal command-line interface for SQL databases`
+	CommandHelpHint       = `hint: try "` + CommandName + ` --help" for more information.`
 	NotConnected          = `(not connected)`
 	HelpPrefix            = `help`
 	QuitPrefix            = `quit`
@@ -79,15 +85,25 @@ var (
 		`tableattr`: `Table attributes unset.`,
 		`title`:     `Title is unset.`,
 	}
-	TimingSet            = `Timing is %s.`
-	TimingDesc           = `Time: %0.3f ms`
-	InvalidValue         = `invalid -%s value %q: %s`
-	NotSupportedByDriver = `%s not supported by %s driver`
-	RelationNotFound     = `Did not find any relation named "%s".`
-	InvalidOption        = `invalid option %q`
-	NotificationReceived = `Asynchronous notification %q %sreceived from server process with PID %d.`
-	NotificationPayload  = `with payload %q `
-	UnknownShortAlias    = `(unk)`
+	TimingSet              = `Timing is %s.`
+	TimingDesc             = `Time: %0.3f ms`
+	InvalidValue           = `invalid -%s value %q: %s`
+	NotSupportedByDriver   = `%s not supported by %s driver`
+	RelationNotFound       = `Did not find any relation named "%s".`
+	InvalidOption          = `invalid option %q`
+	NotificationReceived   = `Asynchronous notification %q %sreceived from server process with PID %d.`
+	NotificationPayload    = `with payload %q `
+	UnknownShortAlias      = `(unk)`
+	InvalidNamedConnection = `warning: named connection %q was not defined: %v`
+	UsageTemplate          = `Usage:
+  {{.UseLine}}
+
+Arguments:
+  DSN   database url or connection name
+
+Flags:
+{{.LocalFlags.FlagUsages | trimTrailingWhitespaces}}
+`
 )
 
 func init() {
@@ -118,18 +134,22 @@ var CommandUpper = func() string {
 	return strings.ToUpper(Command())
 }
 
-// UsageTemplate returns the usage template.
-var UsageTemplate = func() string {
-	n := CommandLower()
-	return n + `, ` + Banner + `
+// Short returns the command name and banner.
+var Short = func() string {
+	return Command() + ", " + Banner
+}
 
-Usage:
-  ` + n + ` [OPTIONS]... [DSN]
+// Logo is the logo.
+var Logo image.Image
 
-Arguments:
-  DSN                            database url
+// LogoPng is the embedded logo.
+//
+//go:embed logo.png
+var LogoPng []byte
 
-{{if .Context.Flags}}\
-Options:
-{{.Context.Flags|FlagsToTwoColumns|FormatTwoColumns}}{{end}}`
+func init() {
+	var err error
+	if Logo, err = png.Decode(bytes.NewReader(LogoPng)); err != nil {
+		panic(err)
+	}
 }
